==> localstack-ext-1.4.1.dev20230321182210.tar.gz: localstack-ext-1.4.1.dev20230321182210/localstack_ext/services/s3/persistence.py <==
_C='tagger.state'
_B='global'
_A='s3'
import glob,logging,os,threading
from functools import singledispatchmethod
from typing import Any
from localstack.aws.api import RequestContext
from localstack.aws.handlers import modify_service_response
from localstack.services.stores import AccountRegionBundle
from localstack.state import StateVisitor
from localstack.state.snapshot import SnapshotPersistencePlugin
from localstack_persistence.snapshot.load import LoadSnapshotVisitor
from localstack_persistence.snapshot.save import SaveSnapshotVisitor
from moto.core import BackendDict
from localstack_ext.bootstrap.pods.server.plugins import StateLifecyclePlugin
LOG=logging.getLogger(__name__)
class S3PersistencePlugin(StateLifecyclePlugin):
	name=_A;service=_A
	def on_after_reset(B):from localstack_ext.services.s3.s3_extended import apply_model_patches as A;A()
class S3SnapshotPersistencePlugin(SnapshotPersistencePlugin):
	name=_A
	def __init__(A):A._dirty_marker=None
	def load(A,*C,**D):from moto.s3.models import s3_backends as B;A._dirty_marker=_S3BucketDirtyMarker(B);modify_service_response.append(_A,A._dirty_marker)
	def create_save_snapshot_visitor(A,service,data_dir):return _SaveS3SnapshotVisitor(service,data_dir,A._dirty_marker)
	def create_load_snapshot_visitor(A,service,data_dir):return _LoadS3SnapshotVisitor(service,data_dir)
class _S3BucketDirtyMarker:
	def __init__(A,backend_dict):A._marked=set();A._mark_lock=threading.RLock();A._backend_dict=backend_dict;A._mark_all()
	def __call__(B,_chain,context,_response):
		A=context
		if not A.service_request:return
		if A.request.method.upper()in['GET','HEAD']:return
		with B._mark_lock:
			if(C:=A.service_request.get('Bucket')):B._marked.add(C)
			if(C:=A.service_request.get('BucketName')):B._marked.add(C)
	def get_and_clear_markers(A):
		with A._mark_lock:B=list(A._marked);A._marked.clear();return B
	def _mark_all(A):
		with A._mark_lock:
			for (D,B) in A._backend_dict.items():C=B[_B];A._marked.update(C.buckets.keys())
class _SaveS3SnapshotVisitor(SaveSnapshotVisitor):
	def __init__(A,service,data_dir,s3_marker):super().__init__(service,data_dir);A.s3_marker=s3_marker
	@singledispatchmethod
	def visit(self,state_container):0
	@visit.register(AccountRegionBundle)
	def _(self,state_container):super()._(state_container)
	@visit.register(BackendDict)
	def _(self,state_container):
		A=self;from moto.s3.models import S3Backend;F=A.s3_marker.get_and_clear_markers()
		for (C,G) in state_container.items():
			D=G[_B];B=os.path.join(A.data_dir,A.service,C,_C);A._encode(D.tagger,B)
			for (E,H) in D.buckets.items():
				if E in F:B=os.path.join(A.data_dir,A.service,C,f"{E}.bucket");A._encode(H,B)
class _LoadS3SnapshotVisitor(LoadSnapshotVisitor):
	@singledispatchmethod
	def visit(self,state_container):0
	@visit.register(BackendDict)
	def _(self,state_container):
		C=state_container;A=self;from localstack_ext.services.s3.s3_mount import get_patched_backend_buckets as E
		for D in glob.glob(os.path.join(A.data_dir,A.service,'*/*.bucket'),recursive=True):B,F=D.split('/')[-2:];G=F[:-7];H=A._deserialize_file(D);I=E(C[B][_B]);I[G]=H
		for B in C.keys():J=os.path.join(A.data_dir,A.service,B,_C);C[B][_B].tagger=A._deserialize_file(J)
	@visit.register(AccountRegionBundle)
	def _(self,state_container):super()._(state_container)

==> localstack-ext-2.0.3.dev20230516084829.tar.gz: localstack-ext-2.0.3.dev20230516084829/localstack_ext/services/s3/persistence.py <==
_B='tagger.state'
_A='global'
import glob,logging,os,threading
from functools import singledispatchmethod
from typing import Any
from localstack.aws.api import RequestContext
from localstack.aws.handlers import modify_service_response
from localstack.services.stores import AccountRegionBundle
from localstack.state import StateVisitor
from localstack.state.snapshot import SnapshotPersistencePlugin
from localstack_persistence.snapshot.load import LoadSnapshotVisitor
from localstack_persistence.snapshot.save import SaveSnapshotVisitor
from moto.core import BackendDict
LOG=logging.getLogger(__name__)
class S3SnapshotPersistencePlugin(SnapshotPersistencePlugin):
	name='s3'
	def __init__(A):A._dirty_marker=None
	def load(A,*C,**D):from moto.s3.models import s3_backends as B;A._dirty_marker=_S3BucketDirtyMarker(B);modify_service_response.append('s3',A._dirty_marker)
	def create_save_snapshot_visitor(A,service,data_dir):return _SaveS3SnapshotVisitor(service,data_dir,A._dirty_marker)
	def create_load_snapshot_visitor(A,service,data_dir):return _LoadS3SnapshotVisitor(service,data_dir)
class _S3BucketDirtyMarker:
	def __init__(A,backend_dict):A._marked=set();A._mark_lock=threading.RLock();A._backend_dict=backend_dict;A._mark_all()
	def __call__(B,_chain,context,_response):
		A=context
		if not A.service_request:return
		if A.request.method.upper()in['GET','HEAD']:return
		with B._mark_lock:
			if(C:=A.service_request.get('Bucket')):B._marked.add(C)
			if(C:=A.service_request.get('BucketName')):B._marked.add(C)
	def get_and_clear_markers(A):
		with A._mark_lock:B=list(A._marked);A._marked.clear();return B
	def _mark_all(A):
		with A._mark_lock:
			for(D,B)in A._backend_dict.items():C=B[_A];A._marked.update(C.buckets.keys())
class _SaveS3SnapshotVisitor(SaveSnapshotVisitor):
	def __init__(A,service,data_dir,s3_marker):super().__init__(service,data_dir);A.s3_marker=s3_marker
	@singledispatchmethod
	def visit(self,state_container):0
	@visit.register(AccountRegionBundle)
	def _(self,state_container):super()._(state_container)
	@visit.register(BackendDict)
	def _(self,state_container):
		A=self;from moto.s3.models import S3Backend;F=A.s3_marker.get_and_clear_markers()
		for(B,G)in state_container.items():
			if B is None:continue
			D=G[_A];C=os.path.join(A.data_dir,A.service,B,_B);A._encode(D.tagger,C)
			for(E,H)in D.buckets.items():
				if E in F:C=os.path.join(A.data_dir,A.service,B,f"{E}.bucket");A._encode(H,C)
class _LoadS3SnapshotVisitor(LoadSnapshotVisitor):
	@singledispatchmethod
	def visit(self,state_container):0
	@visit.register(BackendDict)
	def _(self,state_container):
		C=state_container;A=self;from localstack_ext.services.s3.s3_mount import get_patched_backend_buckets as E
		for D in glob.glob(os.path.join(A.data_dir,A.service,'*/*.bucket'),recursive=True):B,F=D.split('/')[-2:];G=F[:-7];H=A._deserialize_file(D);I=E(C[B][_A]);I[G]=H
		for B in C.keys():J=os.path.join(A.data_dir,A.service,B,_B);C[B][_A].tagger=A._deserialize_file(J)
	@visit.register(AccountRegionBundle)
	def _(self,state_container):super()._(state_container)
